<?php

namespace Tests\Feature;

use Illuminate\Support\Str;
use Laravel\Sanctum\Sanctum;
use Tests\TestCase;
use App\Models\User;
use App\Models\Category;
use Hash;

class CategoryUpdateTest extends TestCase
{
    public function test_success_update_category_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $category = Category::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
            'name' => 'test_name',
        ]);
        $category->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_success_update_category_by_admin_without_change_already_exist_name()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $category = Category::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
            'name' => $category->name
        ]);
        $category->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_update_category_by_admin_without_name_of_category()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $category = Category::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
        ]);
        $category->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'name' => 'The Name field is required.'
        ]);
    }

    public function test_update_category_by_admin_with_already_exist_name()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $category = Category::factory()->create();
        $categoryAlreadyExist = Category::factory()->create();
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
            'name' => $categoryAlreadyExist->name
        ]);
        $category->delete();
        $categoryAlreadyExist->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'name' => 'The Name has already been taken.'
        ]);
    }

    public function test_update_non_existing_category_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/categories/-1?editing=true&editMode=update', [
            'name' => 'test_name'
        ]);
        $user->delete();
        $response->assertStatus(404);
    }

    public function test_success_update_category_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $category = Category::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
            'name' => 'test_name',
        ]);
        $category->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_success_update_category_by_analyst_without_change_already_exist_name()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $category = Category::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
            'name' => $category->name
        ]);
        $category->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_update_category_by_analyst_without_name_of_category()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $category = Category::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
        ]);
        $category->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'name' => 'The Name field is required.'
        ]);
    }

    public function test_update_category_by_analyst_with_already_exist_name()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $category = Category::factory()->create();
        $categoryAlreadyExist = Category::factory()->create();
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
            'name' => $categoryAlreadyExist->name
        ]);
        $category->delete();
        $categoryAlreadyExist->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'name' => 'The Name has already been taken.'
        ]);
    }

    public function test_update_non_existing_category_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/categories/-1?editing=true&editMode=update', [
            'name' => 'test_name'
        ]);
        $user->delete();
        $response->assertStatus(404);
    }

    public function test_update_category_by_common()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $category = Category::factory()->create();
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
            'name' => $category->name
        ]);
        $category->delete();
        $user->delete();
        $response->assertStatus(403);
    }

    public function test_update_category_by_unauthenticate_user()
    {
        $category = Category::factory()->create();
        $response = $this->put('nova-api/categories/'.$category->id.'?editing=true&editMode=update', [
            'name' => $category->name
        ]);
        $category->delete();
        $response->assertStatus(401);
    }
}
